#' Add hydraulic information to a layer of assets
#'
#' @param asset sf object, asset layer.
#' @param hydraulic sf object, hydraulic informations.
#' @param treatment character, how the information should be added.
#' @param s2, should 2 be turned on during operations.
#'
#' @return An sf object with hydraulic informations added.
#' 
#' @export

add_hydraulic = function(
    asset,
    hydraulic,
    treatment = c("max", "mean", "min", "divide"),
    s2 = FALSE
) {
    # Treatment of parameters
    elementary_surface = "bare pitch"
    treatment = match.arg(treatment)
    s2 = !s2 == FALSE
    old_s2 = suppressMessages(sf::sf_use_s2(s2))
    on.exit(suppressMessages(sf::sf_use_s2(old_s2)))

    # internal function (TO BE MOVED?)
    add_value = function(id, hydraulic, method = treatment) {
        if (length(id) == 0) return(rep(NA, length(hydraulic)))
        method = switch(
            method,
            max = max,
            mean = mean,
            min = min
        )
        apply(hydraulic[id, , drop = FALSE], 2, method)
    }

    # Preparation of layers
    param = names(hydraulic)[
        names(hydraulic) %in% c("height", "duration", "velocity")
    ]
    hydraulic = hydraulic[param]
    if (! "id_asset" %in% names(asset)) {asset["id_asset"] = seq(nrow(asset))}

    if (treatment != "divide") {
        # Apply unit treatment only to unit elements
        unit_pitch = asset[! asset[["elementary"]] %in% elementary_surface, ]
        unit_pitch[["area"]] = 
            unit_pitch[["area_asset"]] =
            sf::st_area(unit_pitch)
        id_intersect = suppressMessages(
            sf::st_intersects(unit_pitch, hydraulic)
        )
        unit_pitch[param] = t(
            sapply(
                id_intersect,
                add_value,
                hydraulic = sf::st_drop_geometry(hydraulic),
                method = treatment
            )
        )

        # Apply surface treatment only to surface elements
        bare_pitch = add_hydraulic(
            asset[asset[["elementary"]] %in% elementary_surface, ],
            hydraulic,
            treatment = "divide",
            s2
        )

        result = rbind(unit_pitch[names(bare_pitch)], bare_pitch)
    }

    if (treatment == "divide") {
        # All elements are treated as surface elements
        result = asset
        result[["area_asset"]] = sf::st_area(result)
        result = suppressWarnings(sf::st_intersection(result, hydraulic))
        result[["area"]] = sf::st_area(result)
        result[["n_bare_pitch"]] =
            result[["n_bare_pitch"]] * result[["area"]] / result[["area_asset"]]
        result[["n_mobil_home"]] =
            result[["n_mobil_home"]] * result[["area"]] / result[["area_asset"]]
        result[["n_chalet"]] =
            result[["n_chalet"]] * result[["area"]] / result[["area_asset"]]
    }

    # CHECK IF NECESSARY
    # geom = attr(result, "sf_column")
    # result = result[c(names(result)[names(result) != geom], geom)]

    rownames(result) = NULL
    return(result)
}
