#' Load information for campsite
#'
#' @description
#' `load_campsite()` load a spatial layer describing campsites. It performs also
#' some transformations.
#' 
#' It categorises elements depending on `category`. It also adds data provided
#' by `campsite` to estimates, for each entity in the campsite layer how much
#' element are present (bare pitch, mobil-home and chalet). This estimation
#' depends on `detail`. If `crs` is provided it changes the crs of the layer
#' accordingly.
#'
#' @param path character, path to asset layer.
#' @param campsite data.frame, essential characteristics of campsites.
#' @param detail character, how detailled are the data provided by the asset
#' layer. See details.
#' @param crs character, detail level at which analysis is done
#'      [TO BE REMOVED].
#' @param category data.frame, category of elementary components. Default to
#' `floodam.campsite::elementary_category`.
#'
#' @returns An sf object with campsites' informations.
#' 
#' @export

load_campsite = function(
    path,
    campsite,
    detail = c("uniform", "zonal", "detail"),
    crs,
    category = floodam.campsite::elementary_category
) {
    if (tools::file_ext(path) == "csv") {
        return(load_campsite_general(path))
    }
    
    detail = match.arg(detail)

    result = sf::read_sf(file.path(path))
    if (!missing(crs)) {
        result = sf::st_transform(result, crs = sf::st_crs(crs))
    }
    result = merge(result, category, all.x = TRUE)

    if (!missing(campsite)) {
        result = merge(result, campsite, all.x = TRUE)
    } else {
        stop("No information on pitches given.")
    }

    result[["area"]] = sf::st_area(result)
    if (detail == "detail") {
        result = add_unit_density(result, "chalet")
        result = add_unit_density(result, "mobil-home")
        result = add_zonal_density(result, "bare pitch")

    }

    if (detail == "zonal") {
        result = add_zonal_density(result, "chalet")
        result = add_zonal_density(result, "mobil-home")
        result = add_zonal_density(result, "bare pitch")
    }

    if (detail == "uniform") {
        result = add_uniform_density(result)
    }

    result[["mobil_home"]] = result[["chalet"]] = result[["bare_pitch"]] = NULL

    return(result)
}

add_unit_density = function(dataset, element) {
    col_e = gsub("[ -]", "_", element)
    col_n = paste("n", col_e, sep = "_")
    dataset[[col_n]] = ifelse(dataset[["elementary"]] == element, 1, 0)

    return(dataset)
}

add_zonal_density = function(dataset, element) {
    col_e = gsub("[ -]", "_", element)
    col_n = paste("n", col_e, sep = "_")
    temp = sf::st_drop_geometry(
        dataset[c("camping", "elementary", "area", col_e)]
    )
    temp = temp[temp[["elementary"]] == element,]
    temp = stats::setNames(
        stats::aggregate(temp["area"], temp["camping"], sum),
        c("camping", "area_temp")
    )
    dataset = merge(dataset, temp, all.x = TRUE)
    dataset[[col_n]] = ifelse(
        dataset[["elementary"]] == element,
        dataset[["area"]] * dataset[[col_e]] / dataset[["area_temp"]],
        0
    )
    dataset[["area_temp"]] = NULL

    return(dataset)
}

add_uniform_density = function(dataset) {
    for (element in c("bare pitch", "mobil_home", "chalet")) {
        col_e = gsub("[ -]", "_", element)
        col_n = paste("n", col_e, sep = "_")
        dataset[[col_n]] = dataset[[col_e]]
    }

    return(dataset)
}
