#' Load information for campsite
#'
#' @description
#' `load_campsite_general()` load general informations on campsites.
#'
#' @param path character, path to general campsite information.
#' @param constitution character, sub-path to constitution information.
#' @param occupation character, sub-path to occupation information.
#' @param rental character, sub-path to rental information.
#' @param synthetic logical, should information be synthetic.
#'
#' @returns TO BE PRECISED
#' 
#' @export

load_campsite_general = function(
    path,
    constitution = "constitution",
    occupation = "occupation",
    rental = "rental",
    synthetic = TRUE
) {
    general =  data.table::fread(
        path,
        data.table = FALSE,
        colClasses = c(
            camping = "character",
            id = "character",
            pitch = "integer",
            mobil_home = "integer",
            chalet = "integer",
            bare_pitch = "integer"
        )
    )

    # Preparing constitution by campsite (id)
    constitution = data.table::fread(
        file.path(dirname(path), constitution, "elementary_constitution.csv"),
        data.table = FALSE,
        colClasses = c(
            camping = "character",
            elementary = "character",
            price = "numeric",
            n = "integer",
            comment = "character"
        )
    )
    constitution = merge(
        constitution,
        general[c("camping", "id")],
        all.x = TRUE
    )
    constitution = split(
        constitution[c("id", "precision", "price", "n", "comment")],
        constitution["id"]
    )

    # constitution[["camping"]] %in% general[["camping"]]
    
    # Preparing occupation by campsite (id)
    path_occupation = file.path(dirname(path), occupation)
    occupation = stats::setNames(
        lapply(
            file.path(path_occupation, dir(path_occupation)),
            data.table::fread,
            data.table = FALSE,
            colClasses = c(week = "character")            
        ),
        gsub(
            "elementary_occupation_",
            "",
            tools::file_path_sans_ext(dir(path_occupation))
        )
    )
    occupation = complete_occupation(occupation, "general")

    # Preparing rental by campsite (id)
    path_rental = file.path(dirname(path), rental)
    rental = stats::setNames(
        lapply(
            file.path(path_rental, dir(path_rental)),
            data.table::fread,
            data.table = FALSE,
            colClasses = c(week = "character")            
        ),
        gsub(
            "elementary_rental_",
            "",
            tools::file_path_sans_ext(dir(path_rental))
        )
    )

    result = list(
        general = general,
        constitution = constitution,
        occupation = occupation,
        rental = rental
    )

    if (isTRUE(synthetic)) {
        

    }
    return(result)
}

complete_occupation = function(dataset, base) {
    complete_NA = function(x, base) {
        selection = is.na(x)
        x[selection] = base[selection]
        return(x)
    }

    if (is.list(dataset) & ! is.data.frame(dataset)) {
        if (length(base) == 1) base = dataset[["base"]][[base]]
        return(lapply(dataset, complete_occupation, base))
    }

    selection = names(dataset)[
        ! names(dataset) %in% c("camping", "week", "general")
    ]

    # First complete general column with base (if necessary)
    dataset[["general"]] = complete_NA(dataset[["general"]], base)

    # Then use general column to complete other columns
    dataset[selection] = lapply(
        dataset[selection],
        complete_NA,
        dataset[["general"]]
    )
    return(dataset)
}